
def occultation_info(planet,ground,epoch,frame,sc,DATA_ID,DATA_DIR,Threshold_Surface,EPHE_DIR):
    import spiceypy
    import numpy as np
    import os
    import matplotlib
    from PIL import Image
    import matplotlib.pyplot as plt
    from mpl_toolkits.mplot3d import Axes3D
    
    ang = np.zeros(len(epoch))
    utc = []
    emissn= np.zeros(len(epoch))
    phase = np.zeros(len(epoch)) 
    lat_centric = np.zeros(len(epoch))
    lon_centric = np.zeros(len(epoch))  
    ssl_lat = np.zeros(len(epoch))
    ssl_lon = np.zeros(len(epoch))
    MARS_LS = np.zeros(len(epoch))
    pnear_radius = np.zeros(len(epoch))
    v_mnp_sp = np.zeros((len(epoch),3))
    dist_planet_sc = np.zeros(len(epoch))
    ampm = []
    hr_ang = []
    solar  = np.zeros(len(epoch))
    vsep   = np.zeros(len(epoch))
    ssc_lon = np.zeros(len(epoch))
    ssc_lat = np.zeros(len(epoch))
    distance1 = np.zeros(len(epoch))
    distance = np.zeros(len(epoch))
    
    stop_index = -999      
    for index in range(len(epoch)):
        
        # ground  = NAIF_GS
        # planet  = NAIF_Planet
        # sc      = NAIF_SC
        # epoch = UTC  [-1]
        # frame = Ref_Frame  
        frame    = 'IAU_MARS'  #'MARSIAU ' #'J2000' IAU_MARS
        abcorr   = 'CN+S' # 'LT+S'
        
       # Convert time to et     
        et = epoch[index]
        #et = spiceypy.spiceypy.utc2et(epoch[index])
       
       #Compute the Ls (Solar longitude)
        MARS_LS[index] = spiceypy.spiceypy.lspcn(planet , et, 'none' )*spiceypy.spiceypy.dpr()
        
        
        # saisons 0 ==> PRINTEMPS hemisphere nord, 90 ==> ETE ..... 
       
        
         # get spacecraft position with respect to Earth in IAU_PLANET
        [sp_pos_earth_iau_planet , ltime2] = spiceypy.spiceypy.spkpos(sc, et,
                                           frame, abcorr, ground) 

        
        # get spacecraft position with respect to Planet in IAU_PLANET. spiceypy.spiceypy.spkpos returns the position of a target body relative to an observing body, optionally corrected for light t (planetary aberration) and stellar aberration.
        et = et-ltime2    
        [sp_pos_planet_iau, ltime] = spiceypy.spiceypy.spkpos(sc, et, 
                                           frame, 'NONE', planet) 
                                       
        
               
        ray = spiceypy.spiceypy.unorm(sp_pos_earth_iau_planet)  # direct ray path without perturbations bw sc and earth

        #ray = spiceypy.spiceypy.unorm(sp_pos_planet_iau)
        #ray2 = spiceypy.spiceypy.unorm(sp_pos_earth_iau_planet2)
        # print(ray,ray2)
        for i in range(len(ray[0])) : ray[0][i] = -ray[0][i]
        dist_planet_sc[index]=np.linalg.norm(sp_pos_earth_iau_planet)
        
        # Get ellipsoid radii in body fixed reference frame (IAU_PLANET)
    
        radii = spiceypy.spiceypy.bodvar(int(planet), 'RADII', 3 ) # MARS RADIUS
                                               
        # Get ray direction needed to compute the planet nearest point: that's the line joining the spacecraft and the Earth center
               
        
       # Get nearest point on ellipsoid to the ray  
       # NOTE: dist is here the distance between the line and the MNP point. 
          
        [ pnear, dist ] = spiceypy.spiceypy.npedln(radii[0], radii[1], radii[2], sp_pos_planet_iau, ray[0])

        # If geometrical occultation detected get the moment at which it occurs : 
        if np.logical_and(dist==0,stop_index==-999)==True:
            stop_index = index-1
            break
        # If no geometrical occultation detected then take the last index : 
        
        if np.logical_and(index==len(epoch)-1,stop_index==-999)==True:
            stop_index = index
        
        
        # print(np.sqrt(sp_pos_planet_iau[0]**2+sp_pos_planet_iau[1]**2+sp_pos_planet_iau[2]**2))
        # print(radii)
         
        distance1[index] = dist
        distance = distance1*1e3 + Threshold_Surface
        #print(distance)


       # get Earth position with respect to Sun in J2000
        [Sun_Earth_j2000 , ltime] = spiceypy.spiceypy.spkpos( 'Sun', et,
                                           'J2000', abcorr, 'Earth')  
       
        sunEarthdir =  spiceypy.spiceypy.unorm(Sun_Earth_j2000) 
        #for i in range(len(sunEarthdir[0])-1) : sunEarthdir[0][i] = -sunEarthdir[0][i]
    
        # get S.c position with respect to Sun in J2000
        [earth_sc_j2000 , ltime] = spiceypy.spiceypy.spkpos( 'Earth', et,
                                           'J2000', abcorr, sc)  
       
        Earthspdir =  spiceypy.spiceypy.unorm(earth_sc_j2000) 
       
       
       #angle between SUN-Erath and S/C 
       
        vsep[index] = spiceypy.spiceypy.vsep(sunEarthdir[0], Earthspdir[0])*spiceypy.spiceypy.dpr()
          



        #longitude_sol[index] = pnear
        # approximation impact parameter 
        #[ pnear, dist ] = spiceypy.spiceypy.npedln( rr, rr, rr, sp_pos_planet_iau, ray) ##circular

 
       
       # Compute the lat/lon of the Planet nearest point 
       #spiceypy.spiceypy.recgeo converts rectangular coordinates to geodetic coordinates.
       #spiceypy.spiceypy.reclat converts rectangular (Cartesian) coordinates to Planetocentric / latitudinal coordinates. \
       #All coordinates are expressed as double precision values.
        [pnear_radius[index], longitude, latitude] = spiceypy.spiceypy.reclat(pnear) 
        
        flat = (radii[0] - radii[2])/radii[0]
        [lon, lat, alt] = spiceypy.spiceypy.recgeo(pnear, radii[1], flat)
          
        lon_centric[index]  = longitude * spiceypy.spiceypy.dpr()
        lat_centric[index]   = latitude  * spiceypy.spiceypy.dpr()
        
        lon_geodetic=lon*spiceypy.spiceypy.dpr()
        lat_geodetic=lat*spiceypy.spiceypy.dpr()
        
        if lon_centric[index] <0 :
            lon_centric[index] =360+lon_centric[index] 
        
        if lon_geodetic<0 :
            lon_geodetic=360+lon_geodetic
       
      
      # Compute illumination conditions at planet nearest point
      
        [ phase[index], solar[index], emissn[index]] = spiceypy.spiceypy.illum( planet, et, 'NONE',
                                                   sc, pnear)
    
        #
        # Convert the et value to UTC for human comprehension.
        #
        utc1    = spiceypy.spiceypy.et2utc( et, 'C', 3 )
        utc=np.append(utc,utc1)
        phase[index]  = phase[index]  * spiceypy.spiceypy.dpr()
        solar[index]  = solar[index]  * spiceypy.spiceypy.dpr()
        emissn[index] = emissn[index] * spiceypy.spiceypy.dpr()
          
     
         # Compute the local time from longitude of planet nearest point
         
        [hr, minu, sec, loctime, ampm1] = spiceypy.spiceypy.et2lst( et,   \
                                                      int(planet),  \
                                                      lon_centric[index]/spiceypy.spiceypy.dpr(),'PLANETOCENTRIC')  #'PLANETOGRAPHIC')
        
        ampm = np.append(ampm,ampm1)
        hr_ang1 = hr + minu/60. + sec/3600.
        hr_ang = np.append(hr_ang,hr_ang1)  
        
        #get spacecraft distance to mars nearest point (mnp_dist)
        
        # dist2mnp = spiceypy.spiceypy.unorm(pnear - sp_pos_planet_iau)
             
       
        # mnp_dist = dist2mnp
        # mnp_ls = MARS_LS
        # mnp_sza = solar[index]
        # mnp_phase = phase
        # mnp_loctime = loctime
        # mnp_hour_ang = hr_ang[index]
        
        
        # mnp_lon = longitude  
        # mnp_lat = latitude
        
        
           #Get sub-solar information
           
        [spoint, trgepc, srfvec] = spiceypy.spiceypy.subslr('Near point: ellipsoid', planet, et, frame, 'none', sc)
       
                          
        [ssl_radius, ssl_lon[index], ssl_lat[index]] = spiceypy.spiceypy.reclat(spoint)   #Planetocentric 
        ssl_lon[index]                    = ssl_lon[index] * spiceypy.spiceypy.dpr()
        ssl_lat[index]                    = ssl_lat[index]  * spiceypy.spiceypy.dpr()
    
        if  ssl_lon[index] <0 :
            ssl_lon[index]  =360+ ssl_lon[index]
           
                          
        #Get Spacecraft sub-point information
        
       
        [ssp_pos, ssc_alt] = spiceypy.spiceypy.subpt( 'near point', planet, et, 'none', sc)      
        
        #Convert sub-solar point's rectangular coordinates to
        # planetocentriic  longitude, latitude and altitude. Convert radians
       # to degrees.
           
        [ssc_radius, ssc_lon[index], ssc_lat[index]] = spiceypy.spiceypy.reclat(ssp_pos)   #Planetocentric 
        ssc_lon[index]                    = ssc_lon[index] * spiceypy.spiceypy.dpr()
        ssc_lat[index]                  = ssc_lat[index]  * spiceypy.spiceypy.dpr()
        
        #fprintf( 'Spacecraft sub-point logitude: #s\n: ', ssc_lon[index] )
        #fprintf( 'Spacecraft sub-point latitude: #s\n: ', ssc_lat[index] )
       
       
        #Get angular separation between line-of-sight (ray) and SUN at et
        
        #get SC-SUN direction
       
        [sp_pos_sun , ltime] = spiceypy.spiceypy.spkpos( sc, et,
                                            frame, abcorr, 'SUN')  
                                        
        #sun_dir = spiceypy.spiceypy.unorm(-sp_pos_sun)
       #get angle between +X (ray) and sun_dir
       
        # ang_los_sun = spiceypy.spiceypy.vsep(sun_dir[0], ray[0])* spiceypy.spiceypy.dpr()
           
        
        #Transform these vectors in J2000
       
        
        rotate  = spiceypy.spiceypy.pxform( frame , 'J2000', et )
      
        sp_x_j2000 = rotate.dot(ray[0])
         
         
        #pole = reshape(rotate(:,3,:), 3,[] )
           
          # Get the RADEC coordinates of these vectors
          
          
        [radius, ra, dec] = spiceypy.spiceypy.recrad(sp_x_j2000)
       
        ra  = ra * spiceypy.spiceypy.dpr()
        dec = dec * spiceypy.spiceypy.dpr()
        sp_x_j2000_radec=[]
        sp_x_j2000_radec.append(ra)
        sp_x_j2000_radec.append(dec)
       
        
       
        #Finally, very important: test whether this occultation
        #is a true one: we need to have the MNP point between
        #the spacecraft and the Earth. The case were the spacecraft
        #is between the MNP and the EARTH (no occultation) is
        #not rejected at the efinder level, because it uses
        #only the distance of the Earth/spacecraft line to Mars surface
        #as a criterion
        
        # We really have an occultation only if the MNP is located
            #'between' the Earth  and the spacecraft
            
        #get vector MNP-sp
        #sp_pos_iau = pos_mars
        
        [sp_pos_planet_iau , ltime] = spiceypy.spiceypy.spkpos( sc, et, 
                                            frame, abcorr, planet)  
                                        
         
        [earth_pos_planet_iau, ltime] = spiceypy.spiceypy.spkpos( ground, et,
                                          frame, abcorr, planet)  
      
     
      #get vector MNP-sp
        v_mnp_sp[index]   = -pnear + sp_pos_planet_iau
      
      #get vector MNP-Earth
      
        v_mnp_earth   = -pnear + earth_pos_planet_iau
      
      #get angular separation between MNP-sp and MNP-sun
      # for genuine occultations ang ~ 180 deg.
      #we have to reject the cases where ang ~ 0 deg
      #(here we take 5 deg for the limit, but
      #ang is of the order of 0.01 or 179.9)
        ang[index] = spiceypy.spiceypy.vsep (v_mnp_sp[index], v_mnp_earth) * spiceypy.spiceypy.dpr()
        
                
    ################### FICHIER INFO_MVN......
                
    FILE_OUT = 'INFO_'+DATA_ID+'.txt'
    output = []
    
    
    output.append("Angular separation between MNP-sp and MNP-sun: %s" %(ang[stop_index]))
    output.append('UTC                       : %s' % utc[stop_index])
     
    output.append( 'Emission angle       [deg]: %f' %emissn[stop_index])
    
    output.append( 'Phase angle          [deg]: %f' %phase[stop_index]  )
    
    output.append( 'Lat geocentric       [deg]: %f' %lat_centric[stop_index]  )
    
    output.append( 'Lon geocentric       [deg]: %f' %lon_centric[stop_index] )
    
   
    output.append( 'Sub-Solar Latitude   [deg]: %f' %ssl_lat[stop_index] )
    
    output.append( 'Sub-Solar Longitude  [deg]: %f' %ssl_lon[stop_index] )
    
    
    
    output.append( 'Ls                   [deg]: %f' %MARS_LS[stop_index] )
    
    
    output.append( 'Radius                [km]: %f' %pnear_radius[stop_index] )
    
    output.append( 'S/C to Limb Dist      [km]: %f'   %np.linalg.norm(v_mnp_sp[stop_index]))
    
    output.append( 'S/C to G/S Dist  [10^6 km]: %f' %(dist_planet_sc[stop_index]*1E-6))
    
    output.append( 'local solar time          : %s' %ampm[stop_index] )
    
    output.append( 'local hour angle      [hr]:  %s' %hr_ang[stop_index] )
    
    output.append( 'Solar Zenith angle    [deg]: %f' %solar[stop_index]  )

    output.append( 'Sun-Earth-S/C-Angle   [deg]: %f' %vsep[stop_index]   )
    
    output.append( 'S/C sub-point lon  : %f' %ssc_lon[stop_index] )
   
    output.append( 'S/C sub-point lat  : %f' %ssc_lat[stop_index] )
   
    np.savetxt(EPHE_DIR+'/'+FILE_OUT,output,fmt="%s")
   
    """
    path  = DATA_DIR + '/INFORMATION/mars_magnetism_scale.png'
    path2 = EPHE_DIR + '/trajectory.png'
    
    
    image = Image.open(path)
    fig, ax = plt.subplots()
    ax.imshow(image, extent=(0,360,-90,90))
    ax.set_xlabel("Longitude E [°]")
    ax.set_ylabel("Latitude N [°]")
    x = lon_centric[0:stop_index]
    y = lat_centric[0:stop_index]
    plt.scatter(x[0], y[0], label = 'Start',c='g',s=4,zorder=1)
    plt.scatter(x[-1], y[-1], c='r',s=4, label = 'End',zorder=2)
    plt.plot(x,y,label='Data Trajectory',c='k',zorder=3)

    dimx_box = 60
    dimy_box = 20
    
    if x[0] < 180 : xtext0 = x[0]+ 1*dimx_box
    else : xtext0 = x[0]- 1*dimx_box
    
    if y[0] < 0 : ytext0 = y[0]+ 1*dimy_box
    else : ytext0 = y[0]-1*dimy_box
    
    if x[-1] < 180 : xtext1 = x[-1]+1*dimx_box
    else : xtext1 = x[-1]-1*dimx_box
    
    if y[-1] < 0 : ytext1 = y[-1]+dimy_box
    else : ytext1 = y[-1]-dimy_box

    if xtext0<xtext1:

        if xtext0+1/2*dimx_box > xtext1 - 1/2*dimx_box:
            ecart = abs(xtext0+1/2*dimx_box-(xtext1-1/2*dimx_box))
            if xtext0-ecart > 0 :
                xtext0= xtext0-ecart+5
            else : 
                xtext1 = xtext1+ecart+5
        
        if ytext0+1/2*dimy_box > ytext1 - 1/2*dimy_box:
            ecart = abs(ytext0+1/2*dimy_box-(ytext1-1/2*dimy_box))
            if ytext0-ecart > 0 :
                ytext0=ytext0-ecart+5
            else : 
                ytext1 = ytext1+ecart+5
        
    else :         

        if xtext1+1/2*dimx_box > xtext0 - 1/2*dimx_box:
            ecart = abs(xtext1+1/2*dimx_box-(xtext0-1/2*dimx_box))
            if xtext0-ecart > 0 :
                xtext1= xtext1-ecart+5
            else : 
                xtext0 = xtext0+ecart+5
        
        if ytext1+1/2*dimy_box > ytext0 - 1/2*dimy_box:
            ecart = abs(ytext1+1/2*dimy_box-(ytext0-1/2*dimy_box))
            if ytext1-ecart > 0 :
                ytext1=ytext1-ecart+5
            else : 
                ytext0 = ytext0+ecart+5
        
    ax.annotate('SZ=%0.2f'%solar[0],
            xy=(x[0], y[0]),
            xytext=(xtext0,ytext0),
            arrowprops=dict(arrowstyle='->', connectionstyle='arc3,rad=0',color='k'),
            ha='center', va='bottom',
            bbox=dict(boxstyle='round,pad=0.2', fc='white', alpha=0.5),
            color = 'k')
    
    ax.annotate('SZ=%0.2f'%solar[stop_index],
            xy=(x[-1], y[-1]),
            xytext=(xtext1,ytext1),
            arrowprops=dict(arrowstyle='->', connectionstyle='arc3,rad=0',color='k'),
            ha='center', va='bottom',
            bbox=dict(boxstyle='round,pad=0.2', fc='white', alpha=0.5),
            color = 'k')
    
    plt.legend(prop={'size': 4})    
    plt.savefig(path2,dpi=200)
    plt.close()
    

    """
    return distance

 #Profile file name                                  :  "M65RSR0L04_AIX_041391512_05.TAB"
 #Start time                                   [GRT] :  2004-05-18T15:26:42.558
 #Stop time                                    [GRT] :  2004-05-18T15:27:05.598
 #Measurement time (lowest sample)      [GRT - OWLT] :  2004-05-18T15:08:04.584
 #Orbit Number                                       :        414
 #DSN antenna number                                 :         65
 #Ray path direction of geometrical OCC point  [deg] :      81.42
 #Angle from Diametric of geom. OCC point      [deg] :     151.50
 #Latitude (lowest sample)                     [deg] :      52.51
 #Longitude (lowest sample)                    [deg] :     271.95
# Sub-Solar Latitude (lowest sample)           [deg] :      14.15
# Sub-Solar Longitude (lowest sample)          [deg] :     196.66
# Solar Longitude                              [deg] :      35.07
# MOLA Radius (lowest sample)                   [km] :    3383.14
# MOLA Areoid (lowest sample)                   [km] :    3384.90
# Radius (lowest sample)                        [km] :    3384.41
# Sigma Radius (lowest sample)                  [km] :    -999.99
# Pressure (lowest sample)                      [Pa] :     623.12
# Sigma Pressure (lowest sample)                [Pa] :       0.65
# Temperature (lowest sample)                    [K] :     225.33
# Sigma Temperature (lowest sample)              [K] :       0.15
# Upper Boundary Condition for T (low)           [K] :     130.00
# Upper Boundary Condition for T (medium)        [K] :     165.00
# Upper Boundary Condition for T (high)          [K] :     200.00
# S/C to Limb Distance (lowest sample)          [km] :      7681.
# S/C to G/S Distance (lowest sample)     [*10^6 km] : 342.382231
# Local True Solar Time (lowest sample)      [hours] :      17.02
# Solar Zenith Angle (lowest sample)           [deg] :      69.88
# Sun-Earth-S/C-Angle (lowest sample)          [deg] :      39.15
# G/S Elevation Angle (lowest sample)          [deg] :      73.92']
   


#!/usr/bin/env python3

"""
Toolbox.
"""

import enum
from pathlib import Path
from typing import Any, Dict, Generator, List, Optional

import yaml
from colored import attr, fg
from pudb import set_trace as bp  # noqa


class AutoEnum(enum.Enum):
    def _generate_next_value_(  # type: ignore
        name: str, start: int, count: int, last_values: List[str]
    ) -> str:
        return name


class GetAttributes:
    def get_attributes(self) -> Dict[str, Any]:
        return {
            attribute: self.__getattribute__(attribute)
            for attribute in self.__dir__()
            if not attribute.startswith("__")
            and not callable(self.__getattribute__(attribute))
        }


def directories(
    PATH: Path, FILTER: Optional[str] = None
) -> Generator[Path, None, None]:
    """Iterate over the directories of a given path."""
    if FILTER is None:
        FILTER = "*"
    return (P for P in PATH.glob(FILTER) if P.is_dir())


def format_info(message: str) -> str:
    """Format an info message."""
    return f"{fg('yellow')}{message}{attr(0)}"


def format_validation(message: str) -> str:
    """Format a validation message."""
    return f"{fg('green')}{message}{attr(0)}"


def format_error(message: str) -> str:
    """Format an error message."""
    return f"{fg('red')}Error: {message}{attr(0)}"


def print_info(message: str) -> None:
    """Print an info message with a template format."""
    print(format_info(message))


def print_validation(message: str) -> None:
    """Print a validation message with a template format."""
    print(format_validation(message))


def print_error(message: str) -> None:
    """Print an error message with a template format."""
    print(format_error(message))


def raise_error(message: str) -> None:
    """Raise an error with a template format."""
    raise SystemExit(NotImplementedError(format_error(message)))


def form_yes_or_no(question: str, default_no: bool = True) -> bool:
    """
    Single yes or no question without recursion.

    Credit:
        Inspired from
        @icamys commented on 29 Nov 2020 on Github,
        https://gist.github.com/garrettdreyfus/8153571
    """
    choices = " [y/N]: " if default_no else " [Y/n]: "
    default_answer = "n" if default_no else "y"
    reply = str(input(question + choices)).lower().strip() or default_answer
    if reply[:1] == "y":
        return True
    elif reply[:1] == "n":
        return False
    else:
        return not default_no


def yaml_represent_none(self: Any, _: Any) -> Any:
    return self.represent_scalar("tag:yaml.org,2002:null", "")


def yaml_add_representer_none() -> None:
    yaml.add_representer(type(None), yaml_represent_none)



import numpy as np

def ERRORS(N_data,fsup,error,c,R,Threshold_upp,V,e,eps0,me,Bande,CC):

    def H(R,Threshold_Atmneu):
        if R<=Threshold_Atmneu:
            H = 10E3 # m
        else:
            H = 25E3 # m   P.Whiters et al.(2010)
        return H

    error_ref = np.full(N_data, np.nan)
    error_elec = np.full(N_data, np.nan)
    error_neut = np.full(N_data, np.nan)

    if Bande == 'X':
        f = fsup*880./749.
    else:
        f = fsup*240./749.

    kappa = CC

    ######## ERREURS ###########
    for i in range(N_data):

            error_ref[i]  = c*error[i]/(V*f)*np.sqrt(H(R[i],Threshold_upp)/(2*np.pi*R[i]))
            error_elec[i] = 4*np.pi*error[i]*f*c*me*eps0/(V*e**2)*np.sqrt(2*np.pi*25E3/R[i])
            error_neut[i] = c*error[i]/(V*f*kappa)*np.sqrt(10E3/(2*np.pi*R[i]))
    print('\t ERRORS DONE\n')
    return error_ref,error_elec, error_neut



#!/usr/bin/env python3

"""
radiocc
"""

from .model import (
    Bands,
    Config,
    Export,
    Folders,
    FoldersError,
    L2Data,
    Layers,
    LV2Loops,
    MavenData,
    MexData,
    ProcessType,
    ResultsFolders,
    Scenario,
)

__all__ = [
    "Bands",
    "Config",
    "Folders",
    "FoldersError",
    "Export",
    "L2Data",
    "Layers",
    "LV2Loops",
    "MexData",
    "MavenData",
    "ProcessType",
    "ResultsFolders",
    "Scenario",
]

NAME = "radiocc"
VERSION = "0.4.2"



#!/usr/bin/env python3

"""
Parse the folder to be processed.
"""

import re
import time
from pathlib import Path
from typing import Optional

import numpy
import spiceypy as spice
from colored import attr, fg
from nptyping import NDArray
from pudb import set_trace as bp  # noqa

from radiocc import (
    Config,
    L2Data,
    LV2Loops,
    MavenData,
    MexData,
    ProcessType,
    ResultsFolders,
    Scenario,
    constants,
    util,
)

SPICE_FOLDERS = dict(
    SPK=("*.bsp",),
    SCLK=("*.tsc",),
    PCK=("*.tpc", "*.bpc"),
    ORBNUM=("*.orb",),
    LSK=("*.tls",),
    FK=("*.tf",),
    CK=("*.bc",),
)
SPICE_RPATH = dict(MEX="EXTRAS/ANCILLARY/SPICE", MAVEN="SPICE")
LV2_RPATH = "DATA/LEVEL02"
DATA_RPATH_MAVEN = "DATA"
METADATA_SUFFIX_MEX = ".LBL"
STATION_RPATH = "STATION.txt"
START_STATIONS_NAIF_CODE = 399000
DEFAULT_DSN_STATION_NAIF_CODE = 398990
DEFAULT_DSN_STATION_NAME = "NEW NORCIA"
BODIES_RPATH = "PLANETS_SATELLITE.txt"
SPACECRAFT_RPATH = "SPACECRAFT.txt"
DATA_COL_IDX_MEX = dict(
    utc=1, et=3, distance=4, fsup=6, Doppler=11, diff_Doppler=13, sigma_observed=14
)
METADATA_COL_IDX_MVN = dict(
    station=22,
)
DATA_COL_IDX_MVN = dict(
    ertyr=0,
    ertdoy=1,
    ertsec=2,
    fsky=3,
    fresid=4,
)
SPACECRAFT_NAIF_CODES = dict(MEX=-41, MAVEN=-202)


INVALID_SIGMA_OBSERVED = -99999.999999
DEFAULT_ERROR = 1e-2  # 10 mHz Hinson et al. (1999)
DISTANCE_SHIFT_UNIT = 4
UNITS = dict(KILOMETER=1e3, METER=1)


def prepare_directories(CFG: Config, scenario: Scenario) -> None:
    """Prepare directories."""
    # Create directories if they do not exist yet.
    RESULTS = scenario.results(CFG.results)
    (RESULTS / ResultsFolders.DATA.name).mkdir(parents=True, exist_ok=True)
    (RESULTS / ResultsFolders.PLOTS.name).mkdir(parents=True, exist_ok=True)
    (RESULTS / ResultsFolders.EPHEMERIDS.name).mkdir(parents=True, exist_ok=True)


def load_spice_kernels(PROCESS_PATH: Path, FOLDER_TYPE: ProcessType) -> None:
    """Load SPICE kernels."""
    SPICE_PATH = PROCESS_PATH / SPICE_RPATH[FOLDER_TYPE.name]

    # Ensure no conflict with previously loaded kernels.
    spice.kclear()

    # Iterate SPICE bundle folders.
    for FOLDER, FILTERS in SPICE_FOLDERS.items():
        for FILTER in FILTERS:
            FILES = list((SPICE_PATH / FOLDER).glob(FILTER))
            FILES.extend(list((SPICE_PATH / FOLDER).glob(FILTER.upper())))

            # Iterate files in each folder.
            for FILE in FILES:
                spice.furnsh(str(FILE))

    print("      SPICE kernels loaded.")


def detect_folder_type(PROCESS_PATH: Path) -> Optional[ProcessType]:
    """Detect folder type."""
    # Detect if MEX or MAVEN format.
    folder_type: Optional[ProcessType] = None
    print(f"      {PROCESS_PATH.name}")
    if PROCESS_PATH.name[:3] == "MEX" and PROCESS_PATH.name[-4:] == "V1.0":
        folder_type = ProcessType.MEX
    elif PROCESS_PATH.name[:3] == "mvn":
        folder_type = ProcessType.MAVEN

    # Exit if neither MEX nor MAVEN.
    if folder_type is None:
        s = "      "
        print(
            f"{s}Folder {PROCESS_PATH} follows neither MEX nor MAVEN conventions.\n"
            "\n"
            f"{s}MEX format should respect:\n"
            f"{s}  + folder name should start with `MEX`.\n"
            "\n"
            f"{s}MAVEN conventions are:\n"
            f"{s}  + folder name should start with `mvn`.\n"
        )

    return folder_type


def get_data_file(SCENARIO: Scenario, FOLDER_TYPE: ProcessType) -> Optional[Path]:
    """Get the data input file."""
    if FOLDER_TYPE == ProcessType.MEX:
        # Get DATA LEVEL02
        LV2_PATH = SCENARIO.TO_PROCESS / LV2_RPATH
        LV2_ITER = util.directories(LV2_PATH)
        try:
            LV2_LOOP = next(LV2_ITER)
        except StopIteration:
            s = "      "
            print(f"{s}Folder {LV2_PATH} should contain an OPEN or CLOSED loop folder.")
            return None

        # Check if IFMS or DSN.
        LOOP_ITER = util.directories(LV2_LOOP)
        LOOP_OPTION = next(LOOP_ITER)
        if LOOP_OPTION.name == LV2Loops.IFMS.name:
            DATA_FOLDER = LOOP_OPTION / "DP1"
            DATA_FILTER_RAW = "D1"
        elif LOOP_OPTION.name == LV2Loops.DSN.name:
            DATA_FOLDER = LOOP_OPTION / f"DP{SCENARIO.BAND.name}"
            DATA_FILTER_RAW = "DP"
        else:
            s = "      "
            print(
                f"{s}The {LV2_LOOP} folder expect to contain either IFMS or DSN data."
            )
            return None

        print(
            "      Data architecture: "
            f"{LV2_PATH.name}/{LV2_LOOP.name}/{LOOP_OPTION.name}/{DATA_FOLDER.name}"
        )

        # Get TAB filter to select the good input file.
        DATA_FOLDER_FILTER = f"{DATA_FILTER_RAW}{SCENARIO.BAND.name}"
        DATA_FILTER = f"*_{DATA_FOLDER_FILTER}_*.TAB"

    elif FOLDER_TYPE == ProcessType.MAVEN:
        DATA_FOLDER = SCENARIO.TO_PROCESS / DATA_RPATH_MAVEN
        DATA_FILTER = "*.tab"

    else:
        print(
            f"{fg('red')}"
            "      Error: the data input folder is neither MEX nor MAVEN."
            f"{attr(0)}"
        )

    DATA_FILES_ITER = iter(sorted(DATA_FOLDER.glob(DATA_FILTER), reverse=False))

    # Check that the data folder actually contains some files.
    try:
        DATA_FILE = next(DATA_FILES_ITER)
    except StopIteration:
        print(
            f"{fg('red')}"
            f"      Error: the folder {DATA_FOLDER} does not contain any input file "
            f"matching glob: {DATA_FILTER}."
            f"{attr(0)}"
        )
        return None

    return DATA_FILE


def get_metadata_file(
    SCENARIO: Scenario, FOLDER_TYPE: ProcessType, DATA_FILE: Path
) -> Optional[Path]:
    """Get the data input file."""
    # Get the meta data folder.
    if FOLDER_TYPE == ProcessType.MEX:
        METADATA_FILE = DATA_FILE.with_suffix(METADATA_SUFFIX_MEX)
    else:
        METADATA_FILTER = "*.csv"
        METADATA_FILES_ITER = iter(
            sorted(DATA_FILE.parent.glob(METADATA_FILTER), reverse=False)
        )

        # Check that the data folder actually contains the metadata folder.
        try:
            METADATA_FILE = next(METADATA_FILES_ITER)
        except StopIteration:
            print(
                f"{fg('red')}"
                f"      Error: the folder {DATA_FILE.parent} does not contain any input"
                f" metadata file matching glob: {METADATA_FILTER}."
                f"{attr(0)}"
            )
            return None

    return METADATA_FILE


def get_data_and_meta_files(
    SCENARIO: Scenario, FOLDER_TYPE: ProcessType
) -> Optional[L2Data]:
    """Get data file and metadata file."""
    DATA_FILE = get_data_file(SCENARIO, FOLDER_TYPE)

    if DATA_FILE is None:
        return None

    METADATA_FILE = get_metadata_file(SCENARIO, FOLDER_TYPE, DATA_FILE)

    if METADATA_FILE is None:
        return None

    l2_data = L2Data()
    l2_data.FOLDER_TYPE = FOLDER_TYPE
    l2_data.DATA_FILE = DATA_FILE
    l2_data.METADATA_FILE = METADATA_FILE

    return l2_data


def read_input_metadata(l2_data: L2Data) -> L2Data:
    """Read input metadata."""
    if l2_data.FOLDER_TYPE == ProcessType.MEX:
        # Read LBL file.
        with l2_data.METADATA_FILE.open("r") as F:
            LBL_LINES = F.readlines()

        # Get information in LBL file.
        information_in_LBL = dict(DSN_STATION_NUMBER=False, DISTANCE=False)
        for idx, LINE in enumerate(LBL_LINES):
            if "DSN_STATION_NUMBER" in LINE:
                DSN_STATION_NUMBER = int(float(LINE.split("=")[1]))
                DSN_STATION_NAIF_CODE = START_STATIONS_NAIF_CODE + DSN_STATION_NUMBER
                information_in_LBL["DSN_STATION_NUMBER"] = True

            elif any(
                NAME in LINE for NAME in ("DISTANCE", "GEOMETRIC IMPACT PARAMETER")
            ):
                UNIT = re.search('"(.*?)"', LBL_LINES[idx + DISTANCE_SHIFT_UNIT])
                if UNIT:
                    DISTANCE_UNIT = UNIT.group(1)
                else:
                    raise SystemExit(
                        NotImplementedError(
                            f"{fg('red')}"
                            "Error: "
                            "Cannot parse distance unit."
                            f"{attr(0)}"
                        )
                    )

                information_in_LBL["DISTANCE"] = True

            if all(information_in_LBL.values()):
                break

        l2_data.DISTANCE_UNIT = DISTANCE_UNIT

    elif l2_data.FOLDER_TYPE == ProcessType.MAVEN:
        # FIXME
        # mypy issue with numpy untyped call
        DATA_ARRAY = numpy.genfromtxt(
            l2_data.METADATA_FILE,
            dtype=None,
            comments="#",
            delimiter=",",
            skip_header=1,
        )
        STATION_ID = DATA_ARRAY[2][METADATA_COL_IDX_MVN["station"]]
        DSN_STATION_NAIF_CODE = START_STATIONS_NAIF_CODE + STATION_ID

    else:
        print(
            f"{fg('red')}"
            "      Error: the data input folder is neither MEX nor MAVEN."
            f"{attr(0)}"
        )

    l2_data.dsn_station_naif_code = DSN_STATION_NAIF_CODE
    return l2_data


def check_input_info(l2_data: L2Data, INFORMATION_PATH: Path) -> L2Data:
    """Check input data with the information folder."""
    # Read station information file.
    STATION_PATH = INFORMATION_PATH / STATION_RPATH
    with STATION_PATH.open("r") as F:
        STATIONS_LINES = F.readlines()

    # Check station NAIF ID in information file.
    DSN_STATION_UNKNOWN = True
    for LINE in STATIONS_LINES:
        if str(l2_data.dsn_station_naif_code) in LINE:
            DSN_STATION_UNKNOWN = False
            # dsn_station_name = LINE.split(" ")[1]
            break

    if DSN_STATION_UNKNOWN:
        s = "      "
        print(
            f"{s}The station {l2_data.dsn_station_naif_code} has not been found in the "
            f"base {STATION_PATH.resolve()}. The default station "
            f"{DEFAULT_DSN_STATION_NAME} ({DEFAULT_DSN_STATION_NAIF_CODE}) is "
            "considered.\n"
        )
        DSN_STATION_NAIF_CODE = DEFAULT_DSN_STATION_NAIF_CODE
        # dsn_station_name = DEFAULT_DSN_STATION_NAME
        l2_data.dsn_station_naif_code = DSN_STATION_NAIF_CODE
    else:
        print(f"    station found: {l2_data.dsn_station_naif_code}.")

    # Read planets & satellites information file.
    BODIES_PATH = INFORMATION_PATH / BODIES_RPATH
    with BODIES_PATH.open("r") as F:
        BODIES_LINES = F.readlines()

    # Get planet NAIF ID in information file.
    for LINE in BODIES_LINES:
        if constants.Planet in LINE:
            PLANET_NAIF_CODE = int(float(LINE.split()[0]))
            break

    # Read spacecraft information file.
    # SPACECRAFT_PATH = INFORMATION_PATH / SPACECRAFT_RPATH
    # with SPACECRAFT_PATH.open("r") as F:
    #     SPACECRAFT_LINES = F.readlines()

    # # Get spacecraft NAIF ID in information file.
    # for LINE in SPACECRAFT_LINES:
    #     if  in LINE:
    #         SPACRAFT_NAIF_CODE = int(float(LINE.split()[0]))
    #         break
    SPACECRAFT_NAIF_CODE = SPACECRAFT_NAIF_CODES[l2_data.FOLDER_TYPE.name]

    # Export.
    l2_data.PLANET_NAIF_CODE = PLANET_NAIF_CODE
    l2_data.SPACECRAFT_NAIF_CODE = SPACECRAFT_NAIF_CODE
    return l2_data


def read_L2_metadata(l2_data: L2Data, INFORMATION_PATH: Path) -> L2Data:
    """Read L2 data."""
    l2_data = read_input_metadata(l2_data)
    l2_data = check_input_info(l2_data, INFORMATION_PATH)
    return l2_data


def read_tab_data(l2_data: L2Data) -> L2Data:
    """read the tab file."""
    # Read data from TAB file.
    # FIXME
    # mypy issue with numpy untyped call
    DATA_ARRAY = numpy.genfromtxt(l2_data.DATA_FILE, dtype=None, encoding="utf-8")
    DATA_NROWS = DATA_ARRAY.shape[0]

    if l2_data.FOLDER_TYPE == ProcessType.MEX:
        # Initialize individual column arrays.
        et = numpy.zeros(DATA_NROWS)
        utc = [""] * DATA_NROWS
        distance = numpy.zeros(DATA_NROWS)
        Doppler = numpy.zeros(DATA_NROWS)
        diff_Doppler = numpy.zeros(DATA_NROWS)
        error = numpy.zeros(DATA_NROWS)
        validity_sigma_observed = False

        # Fill arrays from TAB_ARRAY.
        fsup = DATA_ARRAY[0][DATA_COL_IDX_MEX["fsup"]]
        for idx, LINE in enumerate(DATA_ARRAY):
            et[idx] = LINE[DATA_COL_IDX_MEX["et"]]
            utc[idx] = LINE[DATA_COL_IDX_MEX["utc"]]
            distance[idx] = LINE[DATA_COL_IDX_MEX["distance"]]
            Doppler[idx] = LINE[DATA_COL_IDX_MEX["Doppler"]]
            diff_Doppler[idx] = LINE[DATA_COL_IDX_MEX["diff_Doppler"]]

            # FIXME:
            # + float comparison (compare with epsilon value)
            SIGMA_OBSERVED = LINE[DATA_COL_IDX_MEX["sigma_observed"]]
            validity_sigma_observed = float(SIGMA_OBSERVED) == INVALID_SIGMA_OBSERVED
            if validity_sigma_observed:
                error[idx] = DEFAULT_ERROR
            else:
                error[idx] = SIGMA_OBSERVED

        distance *= UNITS[l2_data.DISTANCE_UNIT]

        SURFACES_CONDITIONS: NDArray[float] = distance <= constants.Threshold_Surface
        INTEGRAL_CONDITIONS: NDArray[float] = distance <= constants.Threshold_int

        # Export data.
        mex_data = MexData()
        mex_data.ET = et
        mex_data.UTC = utc
        mex_data.DISTANCE = distance
        mex_data.DOPPLER = Doppler
        mex_data.DIFF_DOPPLER = diff_Doppler
        mex_data.ERROR = error
        mex_data.FSUP = fsup
        mex_data.SURFACES_CONDITIONS = SURFACES_CONDITIONS
        mex_data.INTEGRAL_CONDITIONS = INTEGRAL_CONDITIONS
        l2_data.DATA = mex_data

    elif l2_data.FOLDER_TYPE == ProcessType.MAVEN:
        et = numpy.zeros(DATA_NROWS)
        # fsky = numpy.zeros(DATA_NROWS)
        doppler = numpy.zeros(DATA_NROWS)

        for idx, LINE in enumerate(DATA_ARRAY):
            ERTYR = LINE[DATA_COL_IDX_MVN["ertyr"]]
            ERTDOY = LINE[DATA_COL_IDX_MVN["ertdoy"]]
            ERTSEC = LINE[DATA_COL_IDX_MVN["ertsec"]]
            # fsky[idx] = LINE[DATA_COL_IDX_MVN["fsky"]]
            doppler[idx] = LINE[DATA_COL_IDX_MVN["fresid"]]

            time_format = time.strftime("%H:%M:%S", time.gmtime(ERTSEC))
            Utime = str(ERTYR) + "-" + str(ERTDOY) + "T" + str(time_format)
            et[idx] = spice.str2et(Utime)

        # Export data.
        mvn_data = MavenData()
        mvn_data.ET = et
        # data.FSKY = fsky
        mvn_data.DOPPLER = doppler
        l2_data.DATA = mvn_data

    else:
        print(
            f"{fg('red')}"
            "      Error: the data input folder is neither MEX nor MAVEN."
            f"{attr(0)}"
        )

    return l2_data


def read_L2_data(
    CFG: Config, SCENARIO: Scenario, FOLDER_TYPE: ProcessType
) -> Optional[L2Data]:
    """Read L2 data."""
    if FOLDER_TYPE is None:
        return None

    # Get the file containing the inputs.
    l2_data = get_data_and_meta_files(SCENARIO, FOLDER_TYPE)

    if l2_data is None:
        return None

    # Parse the data from the input data file.
    l2_data = read_L2_metadata(l2_data, CFG.information)
    l2_data = read_tab_data(l2_data)

    return l2_data



#!/usr/bin/env python3

"""
Physical constants.
"""

c = 299792458.0  # m s^-1
kB = 1.38064852e-23  # m^2 kg s^-2 K^-1
G = 6.673e-11  # m^3 kg^-1 s^-2
e = 1.6e-19  # A s
eps0 = 8.854e-12
me = 9.109e-31  # kg
C1 = 1.3077e-6  # m kg^-1 s^2 K  "Hinson et al. (1999)"
CC = C1 * kB  # m^3            "K. L. Cahoy(2005)" Refractive Volume
m_bar = 7.221e-26  # kg                "S.Tellmann (2013)"
V = 3.4e3  # m.s^-1
# radius thresholds
Threshold_Cor = 3.9e6  # 4.0e6  # 4.2E6 #4128432         # Wang et al. (2018) ??
B_Cor_Type = "Linear"  # 'Linear' #'Quadratic'        # 'Linear' or 'Quadratic'
Threshold_neut_upp = 3466000  # M.Patzold et al.(2014) + Peter et al.(2014) ==> 80 km
Threshold_Surface = 3386000  # K. L. Cahoy (2005)
Threshold_int = Threshold_Surface + 1e6
# for refracitivity correction
# neutral_upp = 3.45E6
# iono_low = 3.46E6
# temperature boundary condition
T_BC_low = 130  # Ref: Patzol data processing files
T_BC_med = 165  # Ref : Patzol data processing files
T_BC_upp = 200  # Ref : Patzol data processing files

SC = "MEX"  # '-41'
Planet = "MARS"  # '499'

R_Planet = 3.3866e6
M_Planet = 6.4185e23
Ref_frame = "J2000"

Ref_frame_FP = "marsiau"
# Ref_frame = 'IAU_MARS'



# -*- coding: utf-8 -*-
from setuptools import setup

packages = \
['radiocc', 'radiocc.old']

package_data = \
{'': ['*'], 'radiocc': ['assets/*', 'assets/information/*']}

install_requires = \
['PyYAML>=5.4.1,<6.0.0',
 'click>=8.0.1,<9.0.0',
 'colored>=1.4.2,<2.0.0',
 'dotmap>=1.3.24,<2.0.0',
 'envtoml>=0.1.2,<0.2.0',
 'matplotlib>=3.4.3,<4.0.0',
 'nptyping>=1.4.3,<2.0.0',
 'numpy>=1.21.2,<2.0.0',
 'pandas>=1.3.2,<2.0.0',
 'pudb>=2021.1,<2022.0',
 'pytest-sugar>=0.9.4,<0.10.0',
 'pytest>=6.2.5,<7.0.0',
 'ruamel.yaml>=0.17.16,<0.18.0',
 'scipy>=1.7.1,<2.0.0',
 'spiceypy>=4.0.2,<5.0.0',
 'types-PyYAML>=5.4.10,<6.0.0',
 'types-click>=7.1.5,<8.0.0']

entry_points = \
{'console_scripts': ['radiocc = radiocc.cli:main']}

setup_kwargs = {
    'name': 'radiocc',
    'version': '0.4.2',
    'description': 'Radio occultations',
    'long_description': "# radiocc\n\n[![license badge]][license file]\n[![version badge]][repo url]\n[![python badge]][python url]\n[![coverage badge]][coverage url]\n[![pre-commit badge]][pre-commit url]\n\n> Radio occulation\n\n---\n\n[Installation](#installation) |\n[Roadmap](#roadmap) |\n[Contributing](#contributing) |\n[License](#license)\n\n---\n\n## Installation\n\nThe code is still in development, hence you can only get it on\n[ROB Gitlab repo][repo url]\n\nYou need to:\n\n+ clone it\n+ download [poetry][poetry url] (a tool for dependency management and packaging\n  in Python)\n+ run\n  ```sh\n  poetry install\n  ```\n  inside the project to install the dependencies in a virtual environment (it\n  should be done automatically, long time I didn't do it, if it does not let us\n  know and we will do it together)\n+ to check the virtual env is correctly set, use\n  ```sh\n  poetry env info\n  ```\n  and confirm you have something similar to:\n  ```sh\n  Virtualenv\n  Python:         3.9.6\n  Implementation: CPython\n  Path:           /home/greg/.cache/pypoetry/virtualenvs/radiocc-6zeAPCek-py3.9\n  Valid:          True\n  ```\n+ run\n  ```sh\n  poetry shell\n  ```\n  to activate the virtual environment, it can be useful for IDE\n  and linters but not necessary because it is automatically activated at runtime.\n+ create a folder `RESULTS` and a folder `TO_PROCESS` and place in the latter\n  your *MEX* or *MVN* scenarios to be run run `poetry run radiocc` to run the code\n  automatically on the scenarios to be processed.\n\n## Roadmap\n\n+ improve old code for lisibility, portability and testing\n+ improve CLI interface for parameter tuning\n+ provide GUI interface for parameter tuning\n+ provide GUI tool on graphs to set thresholds and corrections\n+ provide a [`pip`][pip url] library and binary installation\n\n## Contributing\n\n### Evaluate code before commiting\n\nWe use the powerful tool [pre-commit][pre-commit url] to automatically run a\nbattery of tests on the code at commit runtime to:\n\n+ avoid pushing code with warnings and errors\n+ unify code formatting between developpers\n\nThe list of tests and hook scripts can be found here.\n\nThey consist in:\n\n+ few git and file-system checks\n+ [`flake8`][flake8 url] (python static linter)\n+ [`isort`][isort url] (python imports linter & fixer)\n+ [`mypy`][mypy url] (python type checker)\n+ [`black`][black url] (python code fixer)\n+ running tests with [`pytest`][pytest url]\n\nYou can install this development environment using\n\n```sh\npoetry run pre-commit install\n```\n\n### Commiting\n\nYou can fork and ask for pull request on the `dev` branch (or any other excepted\nthe `main` branch).\n\nDepending on your access rights, you can also create a new\nbranch from `dev`, develop on it and ask for a merge into the `dev` branch.\n\n## License\n\nLicensed under the [Apache 2.0 license][license file].\n\n[repo url]: https://gitlab-as.oma.be/gregoireh/radiocc\n[pre-commit file]: https://gitlab-as.oma.be/gregoireh/radiocc/-/raw/main/.pre-commit-config.yaml\n[license file]: https://gitlab-as.oma.be/gregoireh/radiocc/-/raw/main/LICENSE\n[license badge]: https://img.shields.io/badge/License-Apache%202.0-blue.svg\n[coverage badge]: https://img.shields.io/badge/coverage-0%25-red\n[coverage url]: https://github.com/pytest-dev/pytest-cov\n[version badge]: https://img.shields.io/badge/version-0.4.2-blue\n[python url]: https://www.python.org/\n[python badge]: https://img.shields.io/badge/python->=3.9,<3.10-blue\n[pre-commit url]: https://pre-commit.com\n[pre-commit badge]: https://img.shields.io/badge/pre--commit-enabled-brightgreen?logo=pre-commit&logoColor=white\n[poetry url]: https://python-poetry.org/docs\n[flake8 url]: https://flake8.pycqa.org/en/latest\n[isort url]: https://github.com/timothycrosley/isort\n[mypy url]: http://mypy-lang.org\n[black url]: https://github.com/psf/black\n[pytest url]: https://docs.pytest.org/en/latest\n[pip url]: https://pip.pypa.io/en/stable/\n",
    'author': 'Ananya Krishnan',
    'author_email': 'ananyakrishnaniiserk@gmail.com',
    'maintainer': None,
    'maintainer_email': None,
    'url': 'https://gitlab-as.oma.be/gregoireh/radiocc',
    'packages': packages,
    'package_data': package_data,
    'install_requires': install_requires,
    'entry_points': entry_points,
    'python_requires': '>=3.9,<3.10',
}


setup(**setup_kwargs)



import numpy as np
import scipy as sc
import spiceypy
from pudb import set_trace as bp


def Occ_Plane_Conversion(N_data, pos_MEX, vel_MEX, pos_GS, vel_GS):

    gamma = np.zeros((N_data,))

    for i in range(N_data):

        vsep = spiceypy.spiceypy.vsep(pos_MEX[i], pos_GS[i])

        gamma[i] = vsep - (sc.pi / 2.0)

    r_MEX = np.zeros((N_data,))

    z_MEX = np.zeros((N_data,))

    z_GS = np.zeros((N_data,))

    for i in range(N_data):

        r_MEX[i] = np.linalg.norm(pos_MEX[i]) * np.cos(gamma[i])

        z_MEX[i] = np.linalg.norm(pos_MEX[i]) * np.sin(gamma[i])

        z_GS[i] = -np.linalg.norm(pos_GS[i])

    minus_pos_GS = np.full(
        (N_data, 3), np.nan
    )  # np.array([[np.nan for j in range(3)] for i in range(N_data)], dtype = float)

    for i in range(N_data):

        minus_pos_GS[i] = -pos_GS[i]

    vr_MEX = np.zeros((N_data,))

    vz_MEX = np.zeros((N_data,))

    vr_GS = np.zeros((N_data,))

    vz_GS = np.zeros((N_data,))

    for i in range(N_data):

        n_vec = np.cross(pos_GS[i], pos_MEX[i])

        r_vec = np.cross(minus_pos_GS[i], n_vec)

        M = 3

        B = np.array(
            [
                [n_vec[0], r_vec[0], minus_pos_GS[i][0], 1, 0, 0],
                [n_vec[1], r_vec[1], minus_pos_GS[i][1], 0, 1, 0],
                [n_vec[2], r_vec[2], minus_pos_GS[i][2], 0, 0, 1],
            ],
            dtype=float,
        )

        G = -1

        while G < M - 1:

            G += 1

            B[G, :] = B[G, :] / B[G, G]

            for J in range(M):

                if J == G:

                    K = G

                else:

                    if K == M - 1:

                        K = 0

                    else:

                        K += 1

                    B[K, :] = B[K, :] - B[K, G] * B[G, :]

        P = B[:, 3:6]

        beta_MEX = (
            vel_MEX[i][0] * P[1, 0] + vel_MEX[i][1] * P[1, 1] + vel_MEX[i][2] * P[1, 2]
        )

        gamma_MEX = (
            vel_MEX[i][0] * P[2, 0] + vel_MEX[i][1] * P[2, 1] + vel_MEX[i][2] * P[2, 2]
        )

        vr_MEX[i] = -np.sqrt(
            (beta_MEX * r_vec[0]) ** 2
            + (beta_MEX * r_vec[1]) ** 2
            + (beta_MEX * r_vec[2]) ** 2
        )

        vz_MEX[i] = np.sqrt(
            (gamma_MEX * minus_pos_GS[i][0]) ** 2
            + (gamma_MEX * minus_pos_GS[i][1]) ** 2
            + (gamma_MEX * minus_pos_GS[i][2]) ** 2
        )

        beta_GS = (
            vel_GS[i][0] * P[1, 0] + vel_GS[i][1] * P[1, 1] + vel_GS[i][2] * P[1, 2]
        )

        gamma_GS = (
            vel_GS[i][0] * P[2, 0] + vel_GS[i][1] * P[2, 1] + vel_GS[i][2] * P[2, 2]
        )

        vr_GS[i] = np.sqrt(
            (beta_GS * r_vec[0]) ** 2
            + (beta_GS * r_vec[1]) ** 2
            + (beta_GS * r_vec[2]) ** 2
        )

        vz_GS[i] = np.sqrt(
            (gamma_GS * minus_pos_GS[i][0]) ** 2
            + (gamma_GS * minus_pos_GS[i][1]) ** 2
            + (gamma_GS * minus_pos_GS[i][2]) ** 2
        )

        delta_s = np.zeros((N_data,))

        beta_e = np.zeros((N_data,))

    for i in range(N_data):

        a = np.array([r_MEX[i], z_MEX[i]], dtype=float)

        b = np.array([0, z_GS[i]], dtype=float)

        diff_vec = a - b

        delta_s[i] = np.arccos(
            np.dot(-b, diff_vec) / np.linalg.norm(-z_GS[i]) / np.linalg.norm(diff_vec)
        )

        beta_e[i] = (np.pi / 2.0) - np.arccos(
            np.dot(-b, diff_vec) / np.linalg.norm(-z_GS[i]) / np.linalg.norm(diff_vec)
        )

    # print('\tR7: Done')
    # print('gamma[3000]:',gamma[3000], 'beta_e[3000]:', beta_e[3000], 'delta_s[3000]:', delta_s[3000])
    return r_MEX, z_MEX, z_GS, vr_MEX, vz_MEX, vr_GS, vz_GS, gamma, beta_e, delta_s


def Occ_Plane_Conversion_2(N_data, pos_MEX, vel_MEX, pos_GS, vel_GS):

    z_vec = np.full(
        (N_data, 3), np.nan
    )  # np.array([[np.nan for j in range(3)] for i in range(N_data)], dtype = float)

    n_vec = np.full(
        (N_data, 3), np.nan
    )  # np.array([[np.nan for j in range(3)] for i in range(N_data)], dtype = float)

    r_vec = np.full(
        (N_data, 3), np.nan
    )  # np.array([[np.nan for j in range(3)] for i in range(N_data)], dtype = float)

    pos_MEX_new = np.full(
        (N_data, 3), np.nan
    )  # np.array([[np.nan for j in range(3)] for i in range(N_data)], dtype = float)

    vel_MEX_new = np.full(
        (N_data, 3), np.nan
    )  # np.array([[np.nan for j in range(3)] for i in range(N_data)], dtype = float)

    pos_GS_new = np.full(
        (N_data, 3), np.nan
    )  # np.array([[np.nan for j in range(3)] for i in range(N_data)], dtype = float)

    vel_GS_new = np.full(
        (N_data, 3), np.nan
    )  # np.array([[np.nan for j in range(3)] for i in range(N_data)], dtype = float)

    r_MEX = np.zeros((N_data,))

    z_MEX = np.zeros((N_data,))

    z_GS = np.zeros((N_data,))

    vr_MEX = np.zeros((N_data,))

    vz_MEX = np.zeros((N_data,))

    vr_GS = np.zeros((N_data,))

    vz_GS = np.zeros((N_data,))

    gamma = np.zeros((N_data,))

    for i in range(N_data):

        vsep = spiceypy.spiceypy.vsep(pos_MEX[i], pos_GS[i])

        gamma[i] = vsep - (sc.pi / 2.0)

        z_vec[i] = -pos_GS[i] / np.linalg.norm(pos_GS[i])

        n_vec[i] = np.cross(pos_MEX[i] / np.linalg.norm(pos_MEX[i]), z_vec[i]) / np.sin(
            (sc.pi / 2.0) - gamma[i]
        )

        r_vec[i] = np.cross(z_vec[i], n_vec[i])

        H_old_to_new = np.array([z_vec[i][:], n_vec[i][:], r_vec[i][:]], dtype=float)

        pos_MEX_new[i] = np.dot(H_old_to_new, pos_MEX[i])

        pos_GS_new[i] = np.dot(H_old_to_new, pos_GS[i])

        vel_MEX_new[i] = np.dot(H_old_to_new, vel_MEX[i])

        vel_GS_new[i] = np.dot(H_old_to_new, vel_GS[i])

        r_MEX[i] = pos_MEX_new[i][2]

        z_MEX[i] = pos_MEX_new[i][0]

        z_GS[i] = pos_GS_new[i][0]

        vr_MEX[i] = vel_MEX_new[i][2]

        vz_MEX[i] = vel_MEX_new[i][0]

        vr_GS[i] = vel_GS_new[i][2]

        vz_GS[i] = vel_GS_new[i][0]

    # print('')
    # print('')
    # print('\tR7: Done')
    # print('')
    # print('')

    return r_MEX, z_MEX, z_GS, vr_MEX, vz_MEX, vr_GS, vz_GS



#!/usr/bin/env python3
# -*- coding: utf-8 -*-
"""
Created on Mon Nov  4 10:56:32 2019

@author: ananya
"""


import numpy as np
from pudb import set_trace as bp


def Bend_dn(N_data, r_MEX, z_MEX, z_GS, vr_MEX, vz_MEX, vr_GS, vz_GS, gamma, beta_e, delta_s, Doppler_debias,Bande,fsup,c):
    # bp()
    a = np.stack((r_MEX, z_MEX, z_GS, vr_MEX, vz_MEX, vr_GS, vz_GS, gamma, beta_e, delta_s, Doppler_debias))
    # np.savetxt("/home/greg/code/oth/radio-occult/r9_dn_py.txt", a)

    delta_r_dn = np.zeros((N_data,))
    beta_r_dn= np.zeros((N_data,))


    #c = 3.0E8   #m s^-1
    if Bande == 'X':
        fs=(fsup)*(880./749.)  # downward X-band
    else :
        fs=(fsup)*(240./749.)  # downward S-band

    for i in range(N_data):

        vrs = vr_MEX[i]
        vzs = vz_MEX[i]

        vrt = vr_GS[i]
        vzt = vz_GS[i]

        rs = r_MEX[i]
        zs = z_MEX[i]
        zt = z_GS[i]

        gammarad = gamma[i]
        betaErad = beta_e[i]
        deltaSrad = delta_s[i]

        DeltaR=0.;
        BetaR=0.;

        #ABetaRall = np.array([np.nan for k in range(100)], dtype = float)
        #ADeltaRall = np.array([np.nan for k in range(100)], dtype = float)
        #DeltaR_loc = np.array([np.nan for k in range(100)], dtype = float)
        #BetaR_loc = np.array([np.nan for k in range(100)], dtype = float)
        epsilon = 999
        stock= 0
        while np.abs(epsilon)>1E-3:

            b11 = vzs*np.cos(betaErad-BetaR)-vrs*np.sin(betaErad-BetaR)

            b12 = vzt*np.sin(deltaSrad-DeltaR)-vrt*np.cos(deltaSrad-DeltaR)

            b21=(np.sqrt((rs**2)+(zs**2)))*np.cos(betaErad-gammarad-BetaR)

            b22=zt*np.cos(deltaSrad-DeltaR)

            k1= c*((Doppler_debias[i])/fs)+vrs*(np.cos(betaErad-BetaR)-np.cos(betaErad))\
                +vzs*(np.sin(betaErad-BetaR)-np.sin(betaErad))\
                -vrt*(np.sin(deltaSrad-DeltaR)-np.sin(deltaSrad))\
                -vzt*(np.cos(deltaSrad-DeltaR)-np.cos(deltaSrad))

            k2=zt*np.sin(deltaSrad-DeltaR)+(np.sqrt(rs**(2)+zs**(2)))*np.sin(betaErad-gammarad-BetaR);

            ABetaR=(k1-b12*k2/b22)/(b11-b21/b22*b12)

            ADeltaR=(k2-b21*ABetaR)/b22

            #ABetaRall[j] = ABetaR

            #ADeltaRall[j] = ADeltaR

            DeltaR=DeltaR+ADeltaR

            BetaR=BetaR+ABetaR

            epsilon = ADeltaR-stock
            stock= ADeltaR



            #DeltaR_loc[j] = DeltaR

            #BetaR_loc[j] = BetaR


        #print('final value: ' + str(DeltaR))
        delta_r_dn[i] = DeltaR
        #delta_r.append(DeltaR)
        #print('delta_r[i]: ' + str(delta_r[i]))

        beta_r_dn[i] = BetaR
        #beta_r.append(BetaR)


    bend_ang_dn = np.zeros((N_data,))

    imp_param_dn = np.zeros((N_data,))


    for i in range(N_data):

        alpha = delta_r_dn[i] + beta_r_dn[i]

        bend_ang_dn[i] = alpha

        a = (np.sqrt(r_MEX[i]**2 + z_MEX[i]**2)) * np.sin(beta_e[i] - beta_r_dn[i] - gamma[i])

        imp_param_dn[i] = a

    #print("Bending angle and Impact parameter: done")
    #print(fs)
    #print (Doppler_debias)
    print('\t BENDING ANGLE DN DONE ')
    #print('beta_r[0]: ' + str(beta_r[-1]))
    #print('delta_r[0]: ' + str(delta_r[-1]))
    #sys.exit()

    return imp_param_dn, bend_ang_dn, delta_r_dn, beta_r_dn
